BixLib = CreateFrame("Frame","BixLib",UIParent)
local B = BixLib

B.split = function(s,sep)
	local t = {}; i=1
	for x in string.gmatch(s,"[^" .. sep .. "]+") do
		t[i] = x
		i = i+1
	end
	return t
end

B.replace = function(str,old,new)
	local parts = B.split(str,old)
	local t = ""
	for i=1,table.getn(parts) do
		if i>1 then t = t .. new end
		t = t .. parts[i]
		i = i + 1
	end
	return t
end

B.SecsToMins = function(seconds,returnStr)
	seconds = math.floor(seconds)
	local secs = (seconds % 60)
	local mins = ((seconds - secs) / 60)
	if returnStr then
		if mins>0 then
			return tostring(mins) .. "m", tostring(secs) .. "s"
		else
			return tostring(secs)
		end
	else
		return mins,secs
	end
end

B.prnt = function(input)
	print("|cFF00FF00Input: |cFFFF0000" .. tostring(input) .. "|cFF00FF00 Type: |cFFFF0000" .. type(input))
end

B.tablesize = function(T)
  local count = 0
  for _ in pairs(T) do count = count + 1 end
  return count
end

B.PlayerHasSpell = function(spell)
	local spellType,spellID = GetSpellBookItemInfo(spell)
	return (GetSpellBookItemInfo(spell) ~= nil)
end

B.GetSpellCost = function(spellID)
	return GetSpellPowerCost(spellID)[1]["cost"], GetSpellPowerCost(spellID)[1]["name"], GetSpellPowerCost(spellID)[1]["minCost"]
end

B.GetSpellIcon = function(spellID)
	local _,_,icon = GetSpellInfo(spellID)
	return icon
end

B.SpellIsReady = function(spell,book)	
	local start,cooldown,_ = GetSpellCooldown(spell)
	return (start == 0)
end

B.IsSpellActive = function(spell,book)
	if book == nil or book == 1 then
		book = "BOOKTYPE_SPELL"
	elseif book == 2 then
		book = "BOOKTYPE_PET"
	end
	_, _, enabled = GetSpellCooldown(spell, book)
	return enabled
end

B.UnitHasBuff = function(unit,buff)
	return (UnitAura(unit,buff) ~= nil)
end

B.SetTexture = function(texturepath,frame)
	frame.texture = frame.texture or frame:CreateTexture(frame:GetName() .. "_texture","BACKGROUND")
	frame.texture:SetTexture(texturepath)
	frame.texture:ClearAllPoints()
	frame.texture:SetAllPoints(frame)
end

B.lib = {}
B.lib.petTypes = {
	["Basilisk"] = false,
	["Bat"] = false,
	["Bear"] = false,
	["Beetle"] = false,
	["Bird of Prey"] = false,
	["Boar"] = false,
	["Carrion Bird"] = false,
	["Cat"] = false,
	["Chimaera"] = true,
	["Clefthoof"] = true,
	["Core Hound"] = true,
	["Crab"] = false,
	["Crane"] = false,
	["Crocolisk"] = false,
	["Devilsaur"] = true,
	["Direhorn"] = false,
	["Dog"] = false,
	["Dragonhawk"] = false,
	["Fox"] = false,
	["Goat"] = false,
	["Gorilla"] = false,
	["Hydra"] = false,
	["Hyena"] = false,
	["Mechanical"] = false,
	["Monkey"] = false,
	["Moth"] = false,
	["Nether Ray"] = false,
	["Oxen"] = false,
	["Porcupine"] = false,
	["Quilen"] = true,
	["Raptor"] = false,
	["Ravager"] = false,
	["Riverbeast"] = false,
	["Rylak"] = true,
	["Scalehide"] = false,
	["Scorpid"] = false,
	["Serpent"] = false,
	["Shale Spider"] = true,
	["Silithid"] = true,
	["Spider"] = false,
	["Spirit Beast"] = true,
	["Sporebat"] = false,
	["Stag"] = false,
	["Tallstrider"] = false,
	["Turtle"] = false,
	["Warp Stalker"] = false,
	["Wasp"] = false,
	["Water Strider"] = true,
	["Wind Serpent"] = false,
	["Wolf"] = false,
	["Worm"] = true
}

B.IsExotic = function(family)
	local isExotic = false
	for fam,ex in pairs(B.lib.petTypes) do
		if fam == family and ex then isExotic = true end
	end
	return isExotic
end
















B.pets = {}

B.pets.models = {
	-- ["familyName"] = {
		-- ["typeName"] = {
			-- ["colorName"] = {creatureID,...},
			-- ["colorName"] = {creatureID,...},
		-- },
	-- },
	
	["Basilisk"] = {
		["Spiny"] = {
			["Fel-Green"] = {104895},
			["Grey"] = {99598},
			["Olive-Green"] = {107965},
			["Orange"] = {103787},
			["Purple"] = {98957},
		},
		["Three-Eyed"] = {
			["Green"] = {18461},
			["Purple"] = {21854},
			["Red"] = {18648},
			["Yellow"] = {20283},
		},
		["Crested"] = {
			["Black/Blue"] = {5935},
			["Black/Jade"] = {35245},
			["Black/Orange"] = {6215},
			["Black/Purple"] = {14227},
			["Blue"] = {690},
			["Blue/Jade"] = {1551},
			["Blue/Orange"] = {44113},
			["Blue/Purple"] = {1552},
			["Burgundy/Blue"] = {8095},
			["Burgundy/Jade"] = {689},
			["Burgundy/Orange"] = {66188},
			["Burgundy/Purple"] = {8302},
			["Green/Blue"] = {1550},
			["Green/Jade"] = {4863},
			["Green/Orange"] = {688},
			["Green/Purple"] = {28203},
			["Orange/Purple/Blue"] = {8138},
			["Orange/Purple/Jade"] = {689},
			["Orange/Purple"] = {11786},
			["Purple/Orange"] = {66448},
			["Purple/Blue"] = {51013},
			["Red/Blue"] = {5990},
			["Red/Jade"] = {5990},
			["Red/Orange"] = {11785},
			["Red/Purple"] = {8120},
			["White/Blue"] = {5420},
			["White/Jade"] = {43981},
			["White/Orange"] = {43339},
			["White/Purple"] = {42543},
			["Yellow/Blue"] = {35750},
			["Yellow/Jade"] = {4728},
			["Yellow/Orange"] = {56650},
			["Yellow/Purple"] = {39186},
		},
	},
	["Bat"] = {
		["Bat"] = {
			["Brown"] = {1512},
			["Ghostly Tan"] = {50328},
			["Purple/Red"] = {1554},
			["Tan"] = {4861},
			["White"] = {10357},
		},
		["Greater"] = {
			["White"] = {75247},
			["Green"] = {75467},
			["Grey/Brown"] = {75466},
			["Red/Black"] = {79895},
			["Tan/Brown"] = {89788},
		},
	},
	["Bear"] = {
		["Bear"] = {
			["Dark"] = {1186},
			["Dark Brown"] = {42713},
			["Diseased Dark"] = {1797},
			["Diseased Dark Brown"] = {39049},
			["Diseased Grey"] = {47204},
			["Diseased Tan"] = {44482},
			["Diseased White"] = {26643},
			["Grey"] = {1196},
			["Tan"] = {822},
			["White"] = {1961},
		},
		["Old"] = {
			["Black"] = {12037},
			["Brown"] = {14344},
			["Dark Brown"] = {50926},
			["Diseased Black"] = {107596},
			["Diseased Brown"] = {14280},
			["Diseased Dark Brown"] = {107617},
			["Diseased Grey"] = {50931},
			["Diseased White"] = {50930},
			["Grey"] = {107595},
			["White"] = {1130},
		},
	},
	["Beetle"] = {
		["Beetle"] = {
			["Black"] = {7269},
			["Blue"] = {5451},
			["Brown"] = {47390},
			["Green"] = {3253},
			["Orange"] = {51061},
			["Purple"] = {5459},
			["Red"] = {10699},
			["White"] = {51063},
			["Yellow"] = {6555},
		},
		["Scarab"] = {
			["Blue/Green"] = {46127},
			["Brown"] = {51053},
			["Dark Blue"] = {46129},
			["Green/Red"] = {51058},
			["Light Blue"] = {46126},
			["Purple/Red"] = {51057},
		},
	},
	["Bird of Prey"] = {
		["Warden Owl"] = {
			["Brown"] = {91269},
			["Dark"] = {89391},
			["Pale"] = {93333},
			["Purple"] = {109809},
		},
		["Owl"] = {
			["Black"] = {44020},
			["Brown"] = {40194},
			["Dark Brown"] = {50345},
			["Ghostly Green"] = {34982},
			["Green"] = {27244},
			["Grey"] = {1995},
			["Purple"] = {50044},
			["Red/Purple"] = {21904},
			["Orange"] = {21891},
			["Sapphire"] = {50343},
			["Spectral"] = {14343},
			["White"] = {7456},
		},
		["Greater Eagle"] = {
			["Brown"] = {97755},
			["Dark Brown"] = {91717},
			["Light Brown"] = {89829},
			["White"] = {91715},
		},
		["Eagle"] = {
			["Bald"] = {24787},
			["Brown"] = {34886},
			["Light Brown"] = {44628},
			["White"] = {50342},
		},
		["Draenor Kaliri"] = {
			["Black"] = {87651},
			["Brown"] = {82354},
			["Red"] = {88587},
		},
		["Parrot"] = {
			["Blue/Purple"] = {32481},
			["Brown"] = {60196},
			["Dark Red"] = {48137},
			["Green/Orange"] = {48450},
			["Green/Yellow"] = {48138},
			["Grey"] = {48451},
			["Purple"] = {48449},
			["Red"] = {48447},
			["White"] = {48448},
		},
		["Axebeak"] = {
			["Black"] = {83436},
			["Blue"] = {85564},
			["Green"] = {91050}, --3
			["Red"] = {85856},
		},
		["Hawk"] = {
			["Black"] = {24172},
			["Brown"] = {24747},
		},
		["Other"] = {
			["Condor"] = {59782},
			["Raven"] = {69349},
			["Seagull"] = {47680},
			["Undead Eagle"] = {50335},
		},
	},
	["Boar"] = {
		["Boar"] = {
			["Black"] = {42719},
			["Brown"] = {50302},
			["Ghostly Undead"] = {50947},
			["Grey"] = {3098},
			["Orange"] = {38141},
			["Undead"] = {50937},
			["Yellow"] = {2966},
		},
		["Greater"] = {
			["Purple"] = {100846},
			["Black"] = {76914},
			["Brown"] = {86850},
			["Dark Brown"] = {107503},
			["Green"] = {89689},
			["Grey"] = {82728},
			["Red"] = {92875},
			["Light Blue"] = {75416},
			["Tan/White"] = {77298},
		},
		["Armored"] = {
			["Black"] = {50949},
			["Brown"] = {36696},
			["Ghostly Brown"] = {6021},
			["Grey"] = {74620},
		},
		["Felboar"] = {
			["Black/Green"] = {16992},
			["Blue/White"] = {50948},
			["Grey/Orange"] = {16863},
			["Red/Black"] = {16879},
			["Yellow/Black"] = {8303},
		},
	},
	["Carrion Bird"] = {
		["Vulture"] = {
			["Blue"] = {462},
			["Brown"] = {199},
			["Ghostly Blue"] = {50803},
			["Red"] = {50804},
			["Black"] = {2970},
			["White"] = {2931},
		},
		["Condor"] = {
			["Blue"] = {50814},
			["Gold"] = {59782},
			["White"] = {14268},
		},
		["Two-Headed"] = {
			["Ghostly Blue"] = {21470},
			["Ghostly White"] = {50813},
			["Ghostly Yellow"] = {50809},
			["Blue"] = {18707},
			["Green"] = {5983},
			["Red"] = {8207},
			["White"] = {50810},
			["Yellow"] = {8299},
		},
		["Teroclaw"] = {
			["Black"] = {77430},
			["Red"] = {79746},
			["Brown"] = {77626},
			["Young Red"] = {77627},
		},
		["Other"] = {
			["Raven"] = {69349},
		},
	},
	["Cat"] = {
		["Cat"] = {
			["Black"] = {35904},
			["Black Spotted"] = {2032},
			["Black Striped"] = {33127},
			["Brown"] = {3035},
			["Dark Striped"] = {2031},
			["Green Striped"] = {47687},
			["Ice"] = {50819},
			["Orange Spotted"] = {62131},
			["Orange Striped"] = {976},
			["White"] = {7430},
			["White Ghost"] = {3619},
			["White Spotted"] = {42286},
			["White Striped"] = {731},
		},
		["Saber"] = {
			["Aqua"] = {10200},
			["Black"] = {47053},
			["Dark Stiped"] = {2175},
			["Ice"] = {7432},
			["Lavender"] = {7434},
			["Orange Spotted"] = {28001},
			["Orange Striped"] = {28297},
			["White"] = {7433},
			["White Spotted"] = {14430},
			["White Striped"] = {42718},
		},
		["Lynx"] = {
			["Brown"] = {38046},
			["Orange"] = {15366},
		},
		["Lion"] = {
			["Black"] = {5828},
			["Gold"] = {5807},
			["Hunched Lioness"] = {50838},
			["Lioness"] = {14222},
			["Maned"] = {50159},
			["White"] = {3475},
		},
		["Hunched"] = {
			["Black"] = {34034},
			["Black Gem"] = {50815},
			["Black Spotted"] = {28404},
			["Black Striped"] = {50833},
			["Brown"] = {51663},
			["Green Striped"] = {50837},
			["Jade"] = {55236},
			["Orange Spotted"] = {50825},
			["Orange Striped"] = {45096},
			["White Spotted"] = {54700},
			["White Striped"] = {54688},
		},
		["Grand"] = {
			["Black/Red"] = {79909},
			["Black"] = {66633},
			["Brown"] = {56272},
			["Green"] = {66622},
			["Grey"] = {66617},
			["Orange Black Nose"] = {66164},
			["Orange Pink Nose"] = {55946},
			["Red"] = {59102},
			["White/Red"] = {79705},
			["White"] = {50850},
		},
		["Nightsaber"] = {
			["Purple"] = {110562},
			["Brown"] = {96268},
			["Grey"] = {100328},
			["White"] = {99749},
		},
	},
	["Chimaera"] = {
		["Chimaera"] = {
			["Beige"] = {8764},
			["Brown"] = {36304},
			["Green"] = {8660},
			["White"] = {7448},
		},
		["Armored"] = {
			["Green"] = {21879},
			["Purple"] = {50348},
			["White"] = {11497},
			["Yellow"] = {21033},
		},
	},
	["Core Hound"] = {
		["Core Hound"] = {
			["Green"] = {21108},
			["Orange"] = {50842},
			["Dark Orange"] = {77927},
			["Yellow"] = {41166},
		},
		["Guardian"] = {
			["Light Blue"] = {17447},
		},
		["Drake"] = {
			["Drakeadon"] = {50839},
		},
	},
	["Clefthoof"] = {
		["Clefthoof"] = {
			["Brown/Tan"] = {17133},
			["Brown"] = {17132},
			["Dark Brown"] = {19183},
			["Grey"] = {22105},
			["Red"] = {18398},
			["Red/Brown"] = {18259},
		},
		["Greater"] = {
			["Blue"] = {78918},
			["Brown"] = {78570},
			["Grey"] = {86000},
			["Red"] = {78572},
			["White"] = {86000},
		},
		["Saddled"] = {
			["Brown"] = {90124},
			["Grey"] = {78528},
			["Red"] = {108181},
			["White"] = {72881},
		},
		["Rhino"] = {
			["Blue"] = {50997},
			["Brown"] = {25488},
			["Grey"] = {50995},
			["White"] = {50993},
		},
	},
	["Crab"] = {
		["Crab"] = {
			["Blue"] = {831},
			["Bronze"] = {33712},
			["Gem"] = {50959},
			["Red"] = {42715},
			["Silver"] = {101624},
			["White"] = {3108},
		},
		["Spiked"] = {
			["Black"] = {40889},
			["Black/Blue"] = {41922},
			["Black/Purple"] = {41922},
			["Blue"] = {44390},
			["Blue/Light Blue"] = {63347},
			["Blue/Purple"] = {41922},
			["Blue/Red"] = {93226},
			["Dark Blue"] = {69384},
			["Purple"] = {74508},
			["Green"] = {50957},
			["Green/Blue"] = {40239},
			["Green/Purple"] = {50955},
			["Blue/Green"] = {72766},
			["Blue/Brown"] = {66472},
			["Orange"] = {46369},
			["Orange/Blue"] = {64436},
			["Orange/Purple"] = {54557},
			["Red"] = {46369},
			["Red/Blue"] = {61090},
			["Red/Purple"] = {61090},
			["Brown"] = {50952},
			["Dark Red/Blue"] = {73166},
			["Brown-Blue"] = {101554},
		},
	},
	["Crane"] = {
		["Crane"] = {
			["Black"] = {59665},
			["Blue"] = {59786},
			["Brown"] = {59787},
			["Green"] = {59105},
			["Pink"] = {50885},
			["White"] = {55015},
		},
	},
	["Crocolisk"] = {
		["Crocolisk"] = {
			["Green"] = {39452},
			["Grey"] = {3110},
			["White"] = {5053},
			["Yellow"] = {1693},
		},
		["Spiked"] = {
			["Blue"] = {51662},
			["Brown"] = {50964},
			["Green"] = {37078},
			["Purple"] = {50967},
			["White"] = {45896},
		},
	},
	["Devilsaur"] = {
		["Devilsaur"] = {
			["Black"] = {6499},
			["Green"] = {32485},
			["Red"] = {50846},
			["White"] = {6498},
		},
		["Zandalari"] = {
			["Black"] = {69148},
			["Blue"] = {70004},
			["Bronze"] = {69993},
			["Green"] = {70006},
			["Light Brown"] = {69218},
		},
		["Armored"] = {
			["Dark"] = {71529},
		},
	},
	["Direhorn"] = {
		["Direhorn"] = {
			["Blue"] = {67576},
			["Brown"] = {70493},--2
			["Green"] = {67576},
			["Pale"] = {70493},--2
			["Red"] = {69193},--2
		},
	},
	["Dog"] = {
		["Mastiff"] = {
			["Black"] = {42722},
			["Brown"] = {37718},
			["Grey"] = {41561},--2
			["Red"] = {58876},--3
		},
		["Demon"] = {
			["Blue"] = {1547},
			["Ghostly Blue"] = {51044},
			["Ghostly Grey"] = {16176},
			["Ghostly White"] = {16178},
			["Grey"] = {22394},
			["Purple"] = {10356},
			["Red"] = {8921},
			["White"] = {51042},
		},
		["Runed Demon"] = {
			["Blue"] = {51045},
			["Green"] = {8597},
			["Black/Orange"] = {29452},
		},
	},
	["Dragonhawk"] = {
		["Dragonhawk"] = {
			["Black"] = {50357},
			["Blue"] = {25867},
			["Grey"] = {},
			["Orange/Red"] = {},
			["Orange"] = {},
			["Red"] = {},
			["Violet"] = {},
			["Yellow"] = {},
		},
	},
	-- [""] = {
		-- [""] = {
			-- [""] = {},
			-- [""] = {},
		-- },
		-- [""] = {
			-- [""] = {},
			-- [""] = {},
		-- },
	-- },
	-- [""] = {
		-- [""] = {
			-- [""] = {},
			-- [""] = {},
		-- },
		-- [""] = {
			-- [""] = {},
			-- [""] = {},
		-- },
	-- },
}

B.pets.abilities = {
	-- Name
	-- [spellID] = {
		-- ["description"] = "Description",
		-- ["type"] = "special/family/bonus/talent",
		-- ["family"] = "Family",
	-- },
	
	-- Stone Scales
	[159733] = {
		["description"] = "The basilisk scales turn into solid stone, reducing damage taken by 50% for 12 sec.",
		["type"] = "special",
		["family"] = "Basilisk",
	},
	-- Rest (Bear)
	[94019] = {
		["description"] = "You command your pet Bear to take a rest.",
		["type"] = "bonus",
		["family"] = "Bear",
	},
}

B.pets.foodtypes = {
	["Bread"] = {
		["icon"] = "inv_misc_food_12",
	},
	["Cheese"] = {
		["icon"] = "inv_misc_food_03",
	},
	["Fish"] = {
		["icon"] = "inv_misc_fish_04",
	},
	["Fruit"] = {
		["icon"] = "inv_misc_food_19",
	},
	["Fungus"] = {
		["icon"] = "inv_mushroom_11",
	},
	["Meat"] = {
		["icon"] = "inv_misc_food_70",
	},
}

B.pets.families = {
	["Basilisk"] = {
		["exotic"] = false,
		["abilities"] = {159733},
		["diet"] = {"Fish","Meat"},
		["icon"] = "inv_pet_ basilisk",
	},
	["Bat"] = {
		["exotic"] = false,
		["abilities"] = {},
		["diet"] = {"Fruit","Fungus","Meat"},
		["icon"] = "ability_hunter_pet_bat",
	},
	["Bear"] = {
		["exotic"] = false,
		["abilities"] = {nil,nil,94019},
		["diet"] = {"Bread","Cheese","Fish","Fruit","Fungus","Meat"},
		["icon"] = "ability_hunter_pet_bear",
	},
	["Beetle"] = {
		["exotic"] = false,
		["abilities"] = {90339},
		["diet"] = {"Fruit","Fungus"},
		["icon"] = "inv_misc_ahnqirajtrinket_01",
	},	
	["Bird of Prey"] = {
		["exotic"] = false,
		["abilities"] = {nil,nil,94022},
		["diet"] = {"Fish","Meat"},
		["icon"] = "ability_hunter_pet_owl",
	},
	["Boar"] = {
		["exotic"] = false,
		["abilities"] = {},
		["diet"] = {"Bread","Cheese","Fish","Fruit","Fungus","Meat"},
		["icon"] = "ability_hunter_pet_boar",
	},
	["Carrion Bird"] = {
		["exotic"] = false,
		["abilities"] = {24423},
		["diet"] = {"Fish","Meat"},
		["icon"] = "ability_hunter_pet_vulture",
	},
	["Cat"] = {
		["exotic"] = false,
		["abilities"] = {nil,nil,24450},
		["diet"] = {"Fish","Meat"},
		["icon"] = "ability_hunter_pet_cat",
	},
	["Chimaera"] = {
		["exotic"] = true,
		["abilities"] = {54644,92380},
		["diet"] = {"Meat"},
		["icon"] = "ability_hunter_pet_chimera",
	},
	["Clefthoof"] = {
		["exotic"] = true,
		["abilities"] = {nil,nil,160057},
		["diet"] = {"Bread","Cheese","Fruit","Fungus"},
		["icon"] = "inv_clefthoofdraenormount_blue",
	},
	["Core Hound"] = {
		["exotic"] = true,
		["abilities"] = {nil,90355,159788},
		["diet"] = {"Meat"},
		["icon"] = "ability_hunter_pet_corehound",
	},
	["Crab"] = {
		["exotic"] = false,
		["abilities"] = {159926},
		["diet"] = {"Bread","Fish","Fruit","Fungus"},
		["icon"] = "ability_hunter_pet_crab",
	},
	["Crane"] = {
		["exotic"] = false,
		["abilities"] = {159931,nil,126259},
		["diet"] = {"Bread","Fish"},
		["icon"] = "inv_pet_crane",
	},
	["Crocolisk"] = {
		["exotic"] = false,
		["abilities"] = {50433},
		["diet"] = {"Fish","Meat"},
		["icon"] = "ability_hunter_pet_crocolisk",
	},
	["Devilsaur"] = {
		["exotic"] = true,
		["abilities"] = {nil,54680,159953},
		["diet"] = {"Meat"},
		["icon"] = "ability_hunter_pet_devilsaur",
	},
	["Direhorn"] = {
		["exotic"] = false,
		["abilities"] = {137798},
		["diet"] = {"Fruit","Fungus"},
		["icon"] = "inv_pet_direhorn",
	},
	["Dog"] = {
		["exotic"] = false,
		["abilities"] = {},
		["diet"] = {"Bread","Cheese","Fish","Fruit","Fungus","Meat"},
		["icon"] = "inv_pet_mastiff",
	},
	["Dragonhawk"] = {
		["exotic"] = false,
		["abilities"] = {},
		["diet"] = {"Fish","Fruit","Meat"},
		["icon"] = "ability_hunter_pet_dragonhawk",
	},
	["Fox"] = {
		["exotic"] = false,
		["abilities"] = {160011,nil,90347},
		["diet"] = {"Fish","Fruit","Meat"},
		["icon"] = "ability_hunter_aspectofthefox",
	},
	["Goat"] = {
		["exotic"] = false,
		["abilities"] = {},
		["diet"] = {"Bread","Cheese","Fish","Fruit","Fungus","Meat"},
		["icon"] = "inv_pet_ goat",
	},
	["Gorilla"] = {
		["exotic"] = false,
		["abilities"] = {},
		["diet"] = {"Bread","Fruit","Fungus"},
		["icon"] = "ability_hunter_pet_gorilla",
	},
	["Hydra"] = {
		["exotic"] = false,
		["abilities"] = {},
		["diet"] = {"Fish","Meat"},
		["icon"] = "trade_archaeology_whitehydrafigurine",
	},
	["Hyena"] = {
		["exotic"] = false,
		["abilities"] = {},
		["diet"] = {"Meat"},
		["icon"] = "ability_hunter_pet_hyena",
	},
	["Mechanical"] = {
		["exotic"] = false,
		["abilities"] = {},
		["diet"] = {"Mechanical Bits"},
		["icon"] = "ability_mount_mechastrider",
	},
	["Monkey"] = {
		["exotic"] = false,
		["abilities"] = {160044},
		["diet"] = {"Fruit","Fungus"},
		["icon"] = "inv_pet_monkey",
	},
	["Moth"] = {
		["exotic"] = false,
		["abilities"] = {159956},
		["diet"] = {"Bread","Cheese","Fruit","Fungus"},
		["icon"] = "ability_hunter_pet_moth",
	},
	["Nether Ray"] = {
		["exotic"] = false,
		["abilities"] = {160452},
		["diet"] = {"Fungus","Meat"},
		["icon"] = "ability_hunter_pet_netherray",
	},
	["Oxen"] = {
		["exotic"] = false,
		["abilities"] = {},
		["diet"] = {"Bread","Cheese","Fruit","Fungus"},
		["icon"] = "ability_mount_yakmount",
	},
	["Porcupine"] = {
		["exotic"] = false,
		["abilities"] = {nil,nil,126364},
		["diet"] = {"Fungus"},
		["icon"] = "inv_pet_porcupine",
	},
	["Quilen"] = {
		["exotic"] = true,
		["abilities"] = {nil,126393,160049},
		["diet"] = {"Fish","Meat"},
		["icon"] = "achievement_moguraid_01",
	},
	["Raptor"] = {
		["exotic"] = false,
		["abilities"] = {},
		["diet"] = {"Meat"},
		["icon"] = "ability_hunter_pet_raptor",
	},
	["Ravager"] = {
		["exotic"] = false,
		["abilities"] = {},
		["diet"] = {"Meat"},
		["icon"] = "ability_hunter_pet_ravager",
	},
	["Riverbeast"] = {
		["exotic"] = false,
		["abilities"] = {160018},
		["diet"] = {"Bread","Fruit","Fungus"},
		["icon"] = "inv_hippo_green",
	},
	["Rylak"] = {
		["exotic"] = true,
		["abilities"] = {160007},
		["diet"] = {"Fish","Meat"},
		["icon"] = "ability_mount_ironchimera",
	},
	["Scalehide"] = {
		["exotic"] = false,
		["abilities"] = {},
		["diet"] = {"Bread","Cheese","Fruit","Fungus"},
		["icon"] = "inv_mushanbeastmount",
	},
	["Scorpid"] = {
		["exotic"] = false,
		["abilities"] = {160060},
		["diet"] = {"Meat"},
		["icon"] = "ability_hunter_pet_scorpid",
	},
	["Serpent"] = {
		["exotic"] = false,
		["abilities"] = {},
		["diet"] = {"Meat"},
		["icon"] = "spell_nature_guardianward",
	},
	["Shale Spider"] = {
		["exotic"] = true,
		["abilities"] = {160063},
		["diet"] = {"Fish","Meat"},
		["icon"] = "inv_pet_ shalespider",
	},
	["Silithid"] = {
		["exotic"] = true,
		["abilities"] = {160065},
		["diet"] = {"Fungus","Meat"},
		["icon"] = "ability_hunter_pet_silithid",
	},
	["Spider"] = {
		["exotic"] = false,
		["abilities"] = {160067},
		["diet"] = {"Meat"},
		["icon"] = "ability_hunter_pet_spider",
	},
	["Spirit Beast"] = {
		["exotic"] = true,
		["abilities"] = {nil,90361,90328},
		["diet"] = {"Fish","Meat"},
		["icon"] = "ability_druid_primalprecision",
	},
	["Sporebat"] = {
		["exotic"] = false,
		["abilities"] = {},
		["diet"] = {"Bread","Cheese","Fruit","Fungus"},
		["icon"] = "ability_hunter_pet_sporebat",
	},
	["Stag"] = {
		["exotic"] = false,
		["abilities"] = {},
		["diet"] = {"Bread","Cheese","Fruit","Fungus"},
		["icon"] = "inv_talbukdraenor_white",
	},
	["Tallstrider"] = {
		["exotic"] = false,
		["abilities"] = {},
		["diet"] = {"Bread","Cheese","Fruit","Fungus"},
		["icon"] = "ability_hunter_pet_tallstrider",
	},
	["Turtle"] = {
		["exotic"] = false,
		["abilities"] = {26064},
		["diet"] = {"Bread","Fish","Fruit","Fungus"},
		["icon"] = "ability_hunter_pet_turtle",
	},
	["Warp Stalker"] = {
		["exotic"] = false,
		["abilities"] = {35346},
		["diet"] = {"Fish","Fruit"},
		["icon"] = "ability_hunter_pet_warpstalker",
	},
	["Wasp"] = {
		["exotic"] = false,
		["abilities"] = {},
		["diet"] = {"Bread","Cheese","Fruit","Fungus"},
		["icon"] = "ability_hunter_pet_wasp",
	},
	["Water Strider"] = {
		["exotic"] = true,
		["abilities"] = {126311},
		["diet"] = {"Fruit","Fungus"},
		["icon"] = "inv_pet_waterstrider",
	},
	["Wind Serpent"] = {
		["exotic"] = false,
		["abilities"] = {},
		["diet"] = {"Bread","Cheese","Fish"},
		["icon"] = "ability_hunter_pet_windserpent",
	},
	["Wolf"] = {
		["exotic"] = false,
		["abilities"] = {},
		["diet"] = {"Meat"},
		["icon"] = "ability_hunter_pet_wolf",
	},
	["Worm"] = {
		["exotic"] = true,
		["abilities"] = {nil,93433},
		["diet"] = {"Bread","Cheese","Fungus"},
		["icon"] = "ability_hunter_pet_worm",
	},
}