local C = Growler_Constants
local L = BixLib
local O = Growler_Options
local S = Growler_Stable

local TimeSinceLastUpdate = 0
local TimeSinceLastPetAttack = 0
local TimeSinceLastPlayerAttack = 0
local TimeSinceLastScreenIconUpdate = 0
local TimeFalling = 0
local WaitForPet = 0
local IsJumping = false
local IsJumpingNotFalling = false
local TipPlayDead = false
local Growler_PetCalled = 4

local scale = 1
local btnPadding = C.btnPadding
local btnSpacing = C.btnSpacing
local barbtnSize = C.barbtnSize * scale
local btnSize = C.btnSize * scale
local petbtnSize = C.petbtnSize * scale
local petbtnOffset = C.petbtnOffset * scale
local siSize = C.siSize
local portraitSize = C.portraitSize

local Growler_InitiateDB = function()
	Growler.DefaultDB = C.DefaultDB
	if GrowlerDB == nil then
		GrowlerDB = Growler.DefaultDB
	end
	for k,v in pairs(Growler.DefaultDB) do
		if GrowlerDB[k] == nil then
			GrowlerDB[k] = v
		end
	end
	for k,v in  pairs(Growler.DefaultDB.options) do
		if GrowlerDB.options[k] == nil then
			GrowlerDB.options[k] = v
		end
	end
	for i=1,55 do
		if GrowlerDB.pets[i] == nil then
			GrowlerDB.pets[i] = {}
			GrowlerDB.pets[i].icon = ""
			GrowlerDB.pets[i].name = ""
			GrowlerDB.pets[i].level = ""
			GrowlerDB.pets[i].family = ""
			GrowlerDB.pets[i].talent = ""
		end
	end
end

local Growler_Setup = function()
	if GrowlerDB.options["savepos"] then
		scale = GrowlerDB.options.barsize
	else
		scale = C.DefaultDB.options.barsize
	end
	btnPadding = C.btnPadding
	btnSpacing = C.btnSpacing
	barbtnSize = C.barbtnSize * scale
	btnSize = C.btnSize * scale
	petbtnSize = C.petbtnSize * scale
	petbtnOffset = C.petbtnOffset * scale
	siSize = C.siSize
	portraitSize = C.portraitSize * scale
	
	Growler.btn = {}
	Growler.petbtn = {}
	Growler.specbtn = {}
	Growler.GrowlMessageSent = false
	Growler.GrowlWasCast = false
	Growler.LastMessageCheck = 0
	Growler.RecievedMessages = {}
	Growler.EnteredWorld = false
	Growler.PetAvailable = false
	
	Growler.HasVisitedStables = function()
		local emptyslots = 0
		for i=1,55 do
			if GrowlerDB.pets[i].icon == "" then
				emptyslots = emptyslots + 1
			end
		end
		
		return emptyslots < 55
	end
	
	Growler.CheckForGroupGrowl = function(unit, spellName, rank, lineID, spellID)
		if not Growler.GrowlMessageSent and IsInGroup() and spellID == 2649 and (UnitPlayerOrPetInRaid(unit) or UnitPlayerOrPetInParty(unit)) then
			Growler.SendAddonMsg("growl;true")
			Growler.GrowlWasCast = true
		end
	end
		
	Growler.SendAddonMsg = function(msg)
		SendAddonMessage("growler",msg,"PARTY")
		SendAddonMessage("growler",msg,"RAID")
	end
	
	Growler.AddonMsgRecieved = function(prefix,msg,channel,sender)
		if prefix == "growler" then
			sender = L.replace(sender,"-","_")
			Growler.RecievedMessages[sender] = msg		
		end
	end
	
	Growler.CheckCombatLogEvent = function(timestamp,event,_,sourceGUID,sourceName,sourceFlags,sourceRaidFlags,destGUID,destName,destFlags,destRaidFlags,spellID,spellName)
		if tostring(sourceGUID) == UnitGUID("pet") and event == "SWING_DAMAGE" then
			TimeSinceLastPetAttack = 0
		elseif tostring(sourceGUID) == UnitGUID("player") and (event == "RANGE_DAMAGE" or event == "SPELL_DAMAGE") then
			TimeSinceLastPlayerAttack = 0
		end
	end
	
	Growler.msg = function(msg,warning)
		local color1 = "|cFFff4433"
		local color2 = "|cFFaa8822"
		print(color1 .. "{{" .. color2 .. "Growler " .. color1 .. "::" .. color2 .. " " .. msg .. color1 .. "}}")
		if warning then PlaySoundKitID(47355) end
		--if warning then PlaySoundKitID(34040) end
	end
	
	Growler.GetActivePetSlot = function()
		local curName,_ = UnitName("pet")
		local curFamily = UnitCreatureFamily("pet")
		local curIcon = GetPetIcon()
		local activepetslot = 0
		for i=1,5 do
			local petIcon, petName, _, petType, _ = GetStablePetInfo(i)
			if petIcon ~= nil and petIcon == curIcon and petName == curName and petType == curFamily then
				activepetslot = i
			end
		end
		return activepetslot
	end
				
	Growler.SetActivePetSlot = function(load)
		if not InCombatLockdown() then
			local curName,_ = UnitName("pet")
			local curFamily = UnitCreatureFamily("pet")
			local curIcon = GetPetIcon()
			local activepetslot = 0
			if load ~= nil then
				activepetslot = GrowlerDB.options.activepetslot
			else
				if curName ~= nil then
					for i=1,5 do
						local petIcon, petName, _, petType, _ = GetStablePetInfo(i)
						if petIcon ~= nil and petIcon == curIcon and petName == curName and petType == curFamily then
							activepetslot = i
						end
					end
					
					GrowlerDB.options.activepetslot = activepetslot
					
					Growler.btn.call:SetAttribute("macrotext1","/cast [pet] Revive Pet\n/castsequence [nopet] reset=3 Call Pet " .. tostring(activepetslot) .. ", Revive Pet")
					Growler.msg("Active pet slot(" .. activepetslot .. ") saved.")
					PlaySoundKitID(13827)
				else
					Growler.msg("Summon a pet to save that as your active slot.",true)
				end
			end
			if activepetslot == 0 and load then
				Growler.msg("You do not have an active pet slot saved. Summon a pet and click the save button.")
			end
		else
			if not load then
				Growler.msg("You cannot change your active pet in combat.",true)
			end
		end
	end
	
	Growler.UpdateSI = function()
		local prior = 0
		local btn = nil
		for k,v in pairs(Growler.btn) do
			if v.active and v.priority > prior then
				prior = v.priority
				btn = v
			end
		end
		for k,v in pairs(Growler.btn) do
			if not v.active or v.priority < prior then
				v.si.hide()
			end
		end
		if prior > 0 and GrowlerDB.options.screenicons then
			if TimeSinceLastScreenIconUpdate > 2.5 then
				if btn.si.active then
					btn.si:SetAlpha(0.3)
					btn.si:SetSize(siSize*0.9,siSize*0.9)
					btn.si.active = false
				else
					btn.si:SetAlpha(0.5)
					btn.si:SetSize(siSize,siSize)
					btn.si.active = true
				end
				TimeSinceLastScreenIconUpdate = 0
			end
		else
			TimeSinceLastScreenIconUpdate = 0
			for k,v in pairs(Growler.btn) do
				v.si.hide()
			end
		end
	end
	
	Growler.CreateSI = function(name,icon)
		local siName = "GrowlerSI_" .. name
		local si = CreateFrame("FRAME", siName, UIParent)
		si.name = siName
		si.lastupdate = 0
		si.active = false
		si:SetSize(siSize,siSize)
		si:SetPoint("CENTER",World,"CENTER",0,150)
		L.SetTexture(icon,si)
		si:SetAlpha(0)
		si.show = function()
			if GrowlerDB.options.screenicons then
				TimeSinceLastScreenIconUpdate = 0
				si:SetAlpha(0.5)
				si:SetSize(siSize,siSize)
				si.active = true
			end
		end
		si.hide = function()
			si.active = false
			si:SetAlpha(0)
		end
		
		return si
	end
		
	Growler.CreateBar = function()
		Growler.bar = CreateFrame("BUTTON", "GrowlerBar", UIParent)
		Growler.bar.movable = false
		Growler.bar:SetMovable(false)
		Growler.bar:SetClampedToScreen(true)
		Growler.bar:RegisterForClicks("AnyDown")
		Growler.bar:SetScript("OnMouseDown", function(self,button) Growler.bar.MouseDown(self,button) end)
		Growler.bar:RegisterForDrag("LeftButton")
		Growler.bar:SetScript("OnDragStart", function(self) Growler.bar.DragStart(self) end)
		Growler.bar:SetScript("OnDragStop", function(self) Growler.bar.DragStop(self) end)
		Growler.bar.DragStart = function(self)
			if Growler.bar.movable then
				Growler.bar:SetMovable(true)
				Growler.bar:StartMoving()
			end
		end
		Growler.bar.DragStop = function(self)
			Growler.bar:StopMovingOrSizing()
			Growler.bar:SetMovable(false)
			Growler.bar.SavePos()
		end
		Growler.bar.MouseDown = function(self, button)
			if button == "RightButton" then
			elseif button == "MiddleButton" then
			end
		end
		local backdrop = {
		  bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
		  -- bgFile = "Interface\\FrameGeneral\\UI-Background-Rock",
		  edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
		  tile = true,
		  tileSize = 2*btnPadding+btnSize,
		  edgeSize = 16,
		  insets = {left = 5,right = 5,top = 5,bottom = 5}
		}
		Growler.bar:SetBackdrop(backdrop)
		Growler.bar:SetBackdropColor(1,1,1,1)
		Growler.bar:SetBackdropBorderColor(1,1,1,1)
		
		
		Growler.bar.resetbtn = CreateFrame("BUTTON","GrowlerBar_PetSlotButton",Growler.bar)
		Growler.bar.resetbtn.size = 15 * scale
		Growler.bar.resetbtn:SetSize(Growler.bar.resetbtn.size,Growler.bar.resetbtn.size-1)
		Growler.bar.resetbtn:SetPoint("TOPRIGHT",Growler.bar,"TOPRIGHT",(-5),(-8))
		Growler.bar.resetbtn.texture_up = "Interface\\WorldMap\\X_Mark_64Grey"
		Growler.bar.resetbtn.texture_down = "Interface\\WorldMap\\X_Mark_64Grey"
		L.SetTexture(Growler.bar.resetbtn.texture_up,Growler.bar.resetbtn)
		Growler.bar.resetbtn:RegisterForDrag("LeftButton")
		Growler.bar.resetbtn:SetScript("OnDragStart", function(self) Growler.bar.DragStart(self) end)
		Growler.bar.resetbtn:SetScript("OnDragStop", function(self) Growler.bar.DragStop(self) end)
		Growler.bar.resetbtn:RegisterForClicks("LeftButtonUp","LeftButtonDown")
		Growler.bar.resetbtn:SetScript("OnClick",function(self,button,down)
			if button == "LeftButton" then
				if down then
					Growler.bar.resetbtn.texture:SetTexture(Growler.bar.resetbtn.texture_down)
				else
					Growler.bar.resetbtn.texture:SetTexture(Growler.bar.resetbtn.texture_up)
					Growler.bar.ResetPos(true)
					Growler.bar.SavePos()
				end
			end
		end)
		Growler.bar.resetbtn.update = function()
		end
		Growler.bar.resetbtn.setTooltipText = function()
			local tt = "|cffffffffReset bar position."
			GameTooltip:SetText(tt)
		end
		Growler.bar.resetbtn.updateTooltip = function()
			if Growler.bar.resetbtn.mouseover and GrowlerDB.options.tooltips then
				Growler.bar.resetbtn.setTooltipText()
			end
		end
		Growler.bar.resetbtn:SetScript("OnEnter",function(self)
			if GrowlerDB.options.tooltips then
				Growler.bar.resetbtn.mouseover = true
				GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
				Growler.bar.resetbtn.setTooltipText()
				GameTooltip:ClearAllPoints()
				GameTooltip:SetPoint("BOTTOMLEFT",Growler.bar.resetbtn,"TOPLEFT",0,0)
				GameTooltip:Show()
				Growler.bar.resetbtn.updateTooltip()
			end
		end)
		Growler.bar.resetbtn:SetScript("OnLeave",function(self)
			if GrowlerDB.options.tooltips then
				Growler.bar.resetbtn.mouseover = false
				GameTooltip:ClearAllPoints()
				GameTooltip:Hide()
			end
		end)
		
		Growler.bar.settingsbtn = CreateFrame("BUTTON","GrowlerBar_PetSlotButton",Growler.bar)
		Growler.bar.settingsbtn.size = 15 * scale
		Growler.bar.settingsbtn:SetSize(Growler.bar.settingsbtn.size,Growler.bar.settingsbtn.size)
		Growler.bar.settingsbtn:SetPoint("BOTTOMRIGHT",Growler.bar,"BOTTOMRIGHT",(-5),10)
		Growler.bar.settingsbtn.texture_up = "Interface\\WorldMap\\Gear_64Grey"
		Growler.bar.settingsbtn.texture_down = "Interface\\WorldMap\\Gear_64Grey"
		L.SetTexture(Growler.bar.settingsbtn.texture_up,Growler.bar.settingsbtn)
		Growler.bar.settingsbtn:RegisterForDrag("LeftButton")
		Growler.bar.settingsbtn:SetScript("OnDragStart", function(self) Growler.bar.DragStart(self) end)
		Growler.bar.settingsbtn:SetScript("OnDragStop", function(self) Growler.bar.DragStop(self) end)
		Growler.bar.settingsbtn:RegisterForClicks("LeftButtonUp","LeftButtonDown")
		Growler.bar.settingsbtn:SetScript("OnClick",function(self,button,down)
			if button == "LeftButton" then
				if down then
					Growler.bar.settingsbtn.texture:SetTexture(Growler.bar.settingsbtn.texture_down)
				else
					Growler.bar.settingsbtn.texture:SetTexture(Growler.bar.settingsbtn.texture_up)
					InterfaceOptionsFrame_OpenToCategory(O)
					InterfaceOptionsFrame_OpenToCategory(O)
				end
			end
		end)
		Growler.bar.settingsbtn.update = function()
		end
		Growler.bar.settingsbtn.setTooltipText = function()
			local tt = "|cffffffffSettings."
			GameTooltip:SetText(tt)
		end
		Growler.bar.settingsbtn.updateTooltip = function()
			if Growler.bar.settingsbtn.mouseover and GrowlerDB.options.tooltips then
				Growler.bar.settingsbtn.setTooltipText()
			end
		end
		Growler.bar.settingsbtn:SetScript("OnEnter",function(self)
			if GrowlerDB.options.tooltips then
				Growler.bar.settingsbtn.mouseover = true
				GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
				Growler.bar.settingsbtn.setTooltipText()
				GameTooltip:ClearAllPoints()
				GameTooltip:SetPoint("BOTTOMLEFT",Growler.bar.settingsbtn,"TOPLEFT",0,0)
				GameTooltip:Show()
				Growler.bar.settingsbtn.updateTooltip()
			end
		end)
		Growler.bar.settingsbtn:SetScript("OnLeave",function(self)
			if GrowlerDB.options.tooltips then
				Growler.bar.settingsbtn.mouseover = false
				GameTooltip:ClearAllPoints()
				GameTooltip:Hide()
			end
		end)
		
		Growler.bar.lockbtn = CreateFrame("BUTTON","GrowlerBar_Lockbutton",Growler.bar)
		Growler.bar.lockbtn.pos = 1
		Growler.bar.lockbtn.size = barbtnSize
		Growler.bar.lockbtn:SetSize(Growler.bar.lockbtn.size,btnSize)
		Growler.bar.lockbtn:SetPoint("RIGHT",Growler.bar,"RIGHT",((-Growler.bar.settingsbtn.size) - btnSpacing) * Growler.bar.lockbtn.pos,0 * scale)
		Growler.bar.lockbtn.texture_up = "Interface\\BUTTONS\\UI-Button-KeyRing"
		Growler.bar.lockbtn.texture_down = "Interface\\BUTTONS\\UI-Button-KeyRing-Down"
		Growler.bar.lockbtn.texture_disabled = ""
		L.SetTexture(Growler.bar.lockbtn.texture_down,Growler.bar.lockbtn)
		Growler.bar.lockbtn.texture:SetPoint("TOPLEFT",Growler.bar.lockbtn,"TOPLEFT",0 * scale,0 * scale)
		Growler.bar.lockbtn.texture:SetPoint("BOTTOMRIGHT",Growler.bar.lockbtn,"BOTTOMRIGHT",16 * scale,(-18) * scale)
		Growler.bar.lockbtn:RegisterForDrag("LeftButton")
		Growler.bar.lockbtn:SetScript("OnDragStart", function(self) Growler.bar.DragStart(self) end)
		Growler.bar.lockbtn:SetScript("OnDragStop", function(self) Growler.bar.DragStop(self) end)
		Growler.bar.lockbtn:RegisterForClicks("LeftButtonUp")
		Growler.bar.lockbtn:SetScript("OnMouseUp",function(self,button)
			if button == "LeftButton" then
				PlaySoundKitID(8939)
				if Growler.bar.movable then 
					Growler.bar:SetBackdropBorderColor(1,1,1,1)	
					Growler.bar.lockbtn.texture:SetTexture(Growler.bar.lockbtn.texture_down)
					Growler.bar.movable = false
				else
					Growler.bar:SetBackdropBorderColor(0.7,1,0.7,1)
					Growler.bar.lockbtn.texture:SetTexture(Growler.bar.lockbtn.texture_up)
					Growler.bar.movable = true
				end
				Growler.bar.lockbtn.setTooltipText()
			end
		end)
		Growler.bar.lockbtn.update = function()
		end
		Growler.bar.lockbtn.setTooltipText = function()
			local tt = "|cffff0000Lock |cffffffffthe bar from dragging."
			if not Growler.bar.movable then
				tt = "|cff00ff00Unlock |cffffffffthe bar for dragging."
			end
			GameTooltip:SetText(tt)
		end
		Growler.bar.lockbtn.updateTooltip = function()
			if Growler.bar.lockbtn.mouseover and GrowlerDB.options.tooltips then
				Growler.bar.lockbtn.setTooltipText()
			end
		end
		Growler.bar.lockbtn:SetScript("OnEnter",function(self)
			if GrowlerDB.options.tooltips then
				Growler.bar.lockbtn.mouseover = true
				GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
				Growler.bar.lockbtn.setTooltipText()
				GameTooltip:ClearAllPoints()
				GameTooltip:SetPoint("BOTTOMLEFT",Growler.bar.lockbtn,"TOPLEFT",0,0)
				GameTooltip:Show()
				Growler.bar.lockbtn.updateTooltip()
			end
		end)
		Growler.bar.lockbtn:SetScript("OnLeave",function(self)
			if GrowlerDB.options.tooltips then
				Growler.bar.lockbtn.mouseover = false
				GameTooltip:ClearAllPoints()
				GameTooltip:Hide()
			end
		end)
		
	
		Growler.bar.stablebtn = CreateFrame("BUTTON","GrowlerBar_Lockbutton",Growler.bar)
		Growler.bar.stablebtn.pos = 2
		Growler.bar.stablebtn:SetSize(barbtnSize,btnSize)
		Growler.bar.stablebtn:SetPoint("RIGHT",Growler.bar,"RIGHT",((-Growler.bar.settingsbtn.size) - btnSpacing) * Growler.bar.stablebtn.pos,0 * scale)
		Growler.bar.stablebtn.texture_up = "Interface\\BUTTONS\\UI-MicroButton-Spellbook-Up"
		Growler.bar.stablebtn.texture_down = "Interface\\BUTTONS\\UI-MicroButton-Spellbook-Down"
		Growler.bar.stablebtn.texture_disabled = "Interface\\BUTTONS\\UI-MicroButton-Spellbook-Disabled"
		L.SetTexture(Growler.bar.stablebtn.texture_up,Growler.bar.stablebtn)
		Growler.bar.stablebtn.texture:SetPoint("TOPLEFT",Growler.bar.stablebtn,"TOPLEFT",0 * scale,20 * scale)
		Growler.bar.stablebtn.texture:SetPoint("BOTTOMRIGHT",Growler.bar.stablebtn,"BOTTOMRIGHT",0 * scale,(-2) * scale)
		Growler.bar.stablebtn:RegisterForDrag("LeftButton")
		Growler.bar.stablebtn:SetScript("OnDragStart", function(self) Growler.bar.DragStart(self) end)
		Growler.bar.stablebtn:SetScript("OnDragStop", function(self) Growler.bar.DragStop(self) end)
		Growler.bar.stablebtn:RegisterForClicks("LeftButtonUp")
		Growler.bar.stablebtn:SetScript("OnMouseUp",function(self,button)
			if button == "LeftButton" then
				if not Growler.HasVisitedStables() then
					Growler.msg("Visit the stables to use the stable function. (you only have to do this once)",true)
				else
					if S.IsShown then
						Growler.bar.stablebtn.texture:SetTexture(Growler.bar.stablebtn.texture_up)
						S.HideStable()
					else
						Growler.bar.stablebtn.texture:SetTexture(Growler.bar.stablebtn.texture_down)
						S.ShowStable()
					end
				end
			end
		end)
		Growler.bar.stablebtn.update = function()
			if not Growler.HasVisitedStables() then
				Growler.bar.stablebtn.texture:SetTexture(Growler.bar.stablebtn.texture_disabled)
			else
				if S.IsShown then
					Growler.bar.stablebtn.texture:SetTexture(Growler.bar.stablebtn.texture_down)
				else
					Growler.bar.stablebtn.texture:SetTexture(Growler.bar.stablebtn.texture_up)
				end
			end
		end
		Growler.bar.stablebtn.setTooltipText = function()
			local tt = "|cffffffffBrowse your stables."
			GameTooltip:SetText(tt)
		end
		Growler.bar.stablebtn.updateTooltip = function()
			if Growler.bar.stablebtn.mouseover and GrowlerDB.options.tooltips then
				Growler.bar.stablebtn.setTooltipText()
			end
		end
		Growler.bar.stablebtn:SetScript("OnEnter",function(self)
			if GrowlerDB.options.tooltips then
				Growler.bar.stablebtn.mouseover = true
				GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
				Growler.bar.stablebtn.setTooltipText()
				GameTooltip:ClearAllPoints()
				GameTooltip:SetPoint("BOTTOMLEFT",Growler.bar.stablebtn,"TOPLEFT",0,0)
				GameTooltip:Show()
				Growler.bar.stablebtn.updateTooltip()
			end
		end)
		Growler.bar.stablebtn:SetScript("OnLeave",function(self)
			if GrowlerDB.options.tooltips then
				Growler.bar.stablebtn.mouseover = false
				GameTooltip:ClearAllPoints()
				GameTooltip:Hide()
			end
		end)
		
		
		Growler.bar.petslotbtn = CreateFrame("BUTTON","GrowlerBar_PetSlotButton",Growler.bar)
		Growler.bar.petslotbtn.pos = 3
		Growler.bar.petslotbtn.size = barbtnSize
		Growler.bar.petslotbtn.active = false
		Growler.bar.petslotbtn:SetSize(Growler.bar.petslotbtn.size,btnSize)
		Growler.bar.petslotbtn:SetPoint("RIGHT",Growler.bar,"RIGHT",((-Growler.bar.settingsbtn.size) - btnSpacing) * Growler.bar.petslotbtn.pos,0 * scale)
		Growler.bar.petslotbtn.texture_up = "Interface\\BUTTONS\\UI-MicroButton-Bug-Up"
		Growler.bar.petslotbtn.texture_down = "Interface\\BUTTONS\\UI-MicroButton-Bug-Down"
		Growler.bar.petslotbtn.texture_disabled = "Interface\\BUTTONS\\UI-MicroButton-Bug-Up"
		L.SetTexture(Growler.bar.petslotbtn.texture_disabled,Growler.bar.petslotbtn)
		Growler.bar.petslotbtn.texture:SetPoint("TOPLEFT",Growler.bar.petslotbtn,"TOPLEFT",0 * scale,20 * scale)
		Growler.bar.petslotbtn.texture:SetPoint("BOTTOMRIGHT",Growler.bar.petslotbtn,"BOTTOMRIGHT",0 * scale,(-2) * scale)
		Growler.bar.petslotbtn:RegisterForDrag("LeftButton")
		Growler.bar.petslotbtn:SetScript("OnDragStart", function(self) Growler.bar.DragStart(self) end)
		Growler.bar.petslotbtn:SetScript("OnDragStop", function(self) Growler.bar.DragStop(self) end)
		Growler.bar.petslotbtn:RegisterForClicks("LeftButtonUp")
		Growler.bar.petslotbtn:SetScript("OnMouseUp",function(self,button)
			if button == "LeftButton" then
				local activeslot = Growler.GetActivePetSlot()
				if activeslot ~= GrowlerDB.options.activepetslot then
					Growler.SetActivePetSlot()
				end
			end
		end)
		Growler.bar.petslotbtn.update = function()
			if InCombatLockdown() then
				Growler.bar.petslotbtn.texture:SetDesaturated(1)
			else
				local activeslot = Growler.GetActivePetSlot()
				if activeslot == GrowlerDB.options.activepetslot then
					Growler.bar.petslotbtn.texture:SetDesaturated(nil)
					Growler.bar.petslotbtn.texture:SetTexture(Growler.bar.petslotbtn.texture_down)
					Growler.bar.petslotbtn.active = true
				elseif activeslot == 0 then
					Growler.bar.petslotbtn.texture:SetDesaturated(1)
					Growler.bar.petslotbtn.texture:SetTexture(Growler.bar.petslotbtn.texture_up)
					Growler.bar.petslotbtn.active = false
				else
					Growler.bar.petslotbtn.texture:SetDesaturated(nil)
					Growler.bar.petslotbtn.texture:SetTexture(Growler.bar.petslotbtn.texture_up)
					Growler.bar.petslotbtn.active = false
				end
			end
		end
		Growler.bar.petslotbtn.setTooltipText = function()
			local tt = ""
			if Growler.bar.petslotbtn.active then
				tt = "|cffffffffYour current pet slot(|cff00ff00"..tostring(Growler.GetActivePetSlot()).."|cffffffff) is saved."
			elseif Growler.GetActivePetSlot() ~= 0 then
					tt = "|rLeft-click: |cffffffffSave this slot(|cff00ff00"..tostring(Growler.GetActivePetSlot()).."|cffffffff) as your active pet slot."
			end
			GameTooltip:SetText(tt)
		end
		Growler.bar.petslotbtn.updateTooltip = function()
			if Growler.bar.petslotbtn.mouseover and GrowlerDB.options.tooltips then
				Growler.bar.petslotbtn.setTooltipText()
			end
		end
		Growler.bar.petslotbtn:SetScript("OnEnter",function(self)
			if GrowlerDB.options.tooltips then
				Growler.bar.petslotbtn.mouseover = true
				GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
				Growler.bar.petslotbtn.setTooltipText()
				GameTooltip:ClearAllPoints()
				GameTooltip:SetPoint("BOTTOMLEFT",Growler.bar.petslotbtn,"TOPLEFT",0,0)
				GameTooltip:Show()
				Growler.bar.petslotbtn.updateTooltip()
			end
		end)
		Growler.bar.petslotbtn:SetScript("OnLeave",function(self)
			if GrowlerDB.options.tooltips then
				Growler.bar.petslotbtn.mouseover = false
				GameTooltip:ClearAllPoints()
				GameTooltip:Hide()
			end
		end)
		
		Growler.bar.update = function()
			Growler.bar.resetbtn.update()
			Growler.bar.settingsbtn.update()
			Growler.bar.lockbtn.update()
			Growler.bar.petslotbtn.update()
			Growler.bar.stablebtn.update()
			for slot=1,5 do
				Growler.petbtn[slot].update()
			end
			for spec=1,3 do
				Growler.specbtn[spec].update()
			end
		end
		
		-- PORTRAIT
		-- Growler.bar.PortraitFrame = CreateFrame("Button","Growler_PortaitFrame",Growler.bar,"SecureActionButtonTemplate")
		-- local pf = Growler.bar.PortraitFrame
		-- pf:SetAttribute("type","macro")
		-- pf:SetAttribute("macrotext1","/target pet")
		-- pf:SetSize(portraitSize,portraitSize)
		-- pf:SetPoint("RIGHT",Growler.bar,"LEFT",0,-(portraitSize/4))
		-- pf.portraitframe = pf:CreateTexture("Growler_PortaitFrame_Texture","BACKGROUND")
		-- pf.portraitframe:ClearAllPoints()
		-- pf.portraitframe:SetAllPoints(pf)
		-- pf.portraitframe:SetTexture("Interface\\COMMON\\portrait-ring-withbg")
		-- pf.portraitframe:SetPoint("TOPLEFT",-portraitSize,portraitSize)
		-- pf.portraitframe:SetPoint("BOTTOMRIGHT",portraitSize,-portraitSize)
		-- pf.portrait = pf:CreateTexture("Growler_Portait_Texture","BACKGROUND")
		-- pf.portrait:ClearAllPoints()
		-- pf.portrait:SetAllPoints(pf)
		-- SetPortraitTexture(pf.portrait,"pet")
		
		
				
		Growler.bar.SavePos = function()
			local point, relativeTo, relativePoint, xOfs, yOfs = Growler.bar:GetPoint()
			GrowlerDB.options.point, GrowlerDB.options.relativeto, GrowlerDB.options.relativepoint, GrowlerDB.options.pos_x, GrowlerDB.options.pos_y = point, relativeto, relativePoint, xOfs, yOfs
		end
		
		Growler.bar.ResetPos = function(force)
			if not InCombatLockdown() then
				if force then
					scale = C.DefaultDB.options.barsize
					GrowlerDB.options.barsize = scale
				else
					scale = GrowlerDB.options.barsize
				end
				btnPadding = C.btnPadding
				btnSpacing = C.btnSpacing
				barbtnSize = C.barbtnSize * scale
				btnSize = C.btnSize * scale
				petbtnSize = C.petbtnSize * scale
				petbtnOffset = C.petbtnOffset * scale
				siSize = C.siSize
				portraitSize = C.portraitSize * scale
				
				local spec = GetSpecialization()
				local _, _, _, selected, _, _, _, _  = GetTalentInfo(1, 1, 1)
				local hasLonewolf = (spec == 2 and selected)			
				local numbtns = 0
				
				local point, relativeto, relativepoint, pos_x, pos_y = "CENTER", World, "CENTER", 0, -200
				if GrowlerDB.options.savepos and force ~= true then
					point, relativeto, relativepoint, pos_x, pos_y = GrowlerDB.options.point, GrowlerDB.options.relativeto, GrowlerDB.options.relativepoint, GrowlerDB.options.pos_x, GrowlerDB.options.pos_y
				end
				
				if GrowlerDB.options.enable and GrowlerDB.options.enable_growl then
					numbtns = numbtns + 1
					Growler.btn.growl:Show()
					Growler.btn.growl.barpos = numbtns
				else
					Growler.btn.growl:Hide()
				end
				
				if GrowlerDB.options.enable and (GrowlerDB.options.enable_call or GrowlerDB.options.enable_revive or GrowlerDB.options.enable_mend) then
					numbtns = numbtns + 1
					Growler.btn.call:Show()
					Growler.btn.call.barpos = numbtns
				else
					Growler.btn.call:Hide()
				end
				
				if GrowlerDB.options.enable and (GrowlerDB.options.enable_playdead or GrowlerDB.options.enable_wakeup) then
					numbtns = numbtns + 1
					Growler.btn.playdead:Show()
					Growler.btn.playdead.barpos = numbtns
				else
					Growler.btn.playdead:Hide()
				end
				
				if GrowlerDB.options.enable and GrowlerDB.options.enable_passive then
					numbtns = numbtns + 1
					Growler.btn.passive:Show()
					Growler.btn.passive.barpos = numbtns
				else
					Growler.btn.passive:Hide()
				end
				
				if GrowlerDB.options.enable and GrowlerDB.options.enable_assist then
					numbtns = numbtns + 1
					Growler.btn.assist:Show()
					Growler.btn.assist.barpos = numbtns
				else
					Growler.btn.assist:Hide()
				end
				
				-- local barwidth = ((portraitSize/2) + ((barbtnSize+btnSpacing)*3) + (btnPadding*2) + (numbtns * (btnSize+btnSpacing)))
				local barwidth = (((barbtnSize+btnSpacing)*3) + (btnPadding*2) + (numbtns * (btnSize+btnSpacing)))
				local petbtnwidth = (3+btnSpacing + ((petbtnSize+btnSpacing) * 5))
				local specbtnwidth = (4+btnSpacing + ((petbtnSize+btnSpacing) * 3))
				if GrowlerDB.options.showpets and GrowlerDB.options.showspecs and GrowlerDB.options.petsbottom == GrowlerDB.options.specsbottom and barwidth < (petbtnwidth + specbtnwidth) then
					barwidth = (petbtnwidth + specbtnwidth)
				elseif GrowlerDB.options.showpets and barwidth < petbtnwidth then
					barwidth = petbtnwidth
				elseif GrowlerDB.options.showspecs and barwidth < specbtnwidth then
					barwidth = specbtnwidth
				end
				
				Growler.bar:SetMovable(true)
				Growler.bar:SetUserPlaced(false)
				Growler.bar:ClearAllPoints()
				Growler.bar:SetPoint(point,relativeto,relativepoint,pos_x,pos_y)
				Growler.bar:SetMovable(false)
				
				Growler.bar.resetbtn.size = 15*scale
				Growler.bar.settingsbtn.size = 15*scale
				Growler.bar.resetbtn:SetSize(Growler.bar.resetbtn.size,Growler.bar.resetbtn.size)
				Growler.bar.settingsbtn:SetSize(Growler.bar.settingsbtn.size,Growler.bar.settingsbtn.size)
				Growler.bar.lockbtn:SetSize(barbtnSize,btnSize)
				Growler.bar.stablebtn:SetSize(barbtnSize,btnSize)
				Growler.bar.petslotbtn:SetSize(barbtnSize,btnSize)
				
				for k,btn in pairs(Growler.btn) do
					btn.hide()
					btn:ClearAllPoints()
					btn:SetSize(btnSize,btnSize)
					btn.text:SetTextHeight(16 * scale)
					-- local btnx = (portraitSize/2) + btnPadding + 2 + ((btnSize+btnSpacing) * (btn.barpos-1))
					local btnx = btnPadding + 2 + ((btnSize+btnSpacing) * (btn.barpos-1))
					btn:SetPoint("BOTTOMLEFT",Growler.bar,"BOTTOMLEFT",btnx,btnPadding)
					if hasLonewolf then btn.si:Hide() else btn.si:Show() end
				end
				
				for spec=1,3 do
					local p = spec
					if spec == 3 then p = 1 elseif spec == 1 then p = 3 end
					local point, relativepoint, y_offset = "BOTTOMLEFT", "TOPRIGHT", -petbtnOffset
					if GrowlerDB.options.specsbottom then
						point, relativepoint, y_offset = "TOPLEFT", "BOTTOMRIGHT", -y_offset
					end
					Growler.specbtn[spec]:ClearAllPoints()
					Growler.specbtn[spec]:SetSize(petbtnSize,petbtnSize)
					Growler.specbtn[spec]:SetPoint(point,Growler.bar,relativepoint, -2- ((petbtnSize+btnSpacing) * p), y_offset)
					if hasLonewolf or not GrowlerDB.options.showspecs then Growler.specbtn[spec]:Hide() else Growler.specbtn[spec]:Show() end
				end
				
				for slot=1,5 do
					local point, relativepoint, y_offset = "BOTTOMRIGHT", "TOPLEFT", -petbtnOffset
					if GrowlerDB.options.petsbottom then
						point, relativepoint, y_offset = "TOPRIGHT", "BOTTOMLEFT", -y_offset
					end
					Growler.petbtn[slot]:ClearAllPoints()
					Growler.petbtn[slot]:SetSize(petbtnSize,petbtnSize)
					Growler.petbtn[slot]:SetPoint(point,Growler.bar,relativepoint, 3+((petbtnSize+btnSpacing) * slot), y_offset)
					if hasLonewolf or not GrowlerDB.options.showpets then Growler.petbtn[slot]:Hide() else Growler.petbtn[slot]:Show() end
				end
				
				
				Growler.bar:SetSize(barwidth, btnSize + (btnPadding*2))
				
				-- PORTRAIT
				-- Growler.bar.PortraitFrame:SetSize(portraitSize,portraitSize)
				-- Growler.bar.PortraitFrame:SetPoint("RIGHT",Growler.bar,"LEFT",(portraitSize/2),-(portraitSize/4)+2*scale)
				-- Growler.bar.PortraitFrame.portraitframe:SetPoint("TOPLEFT",-portraitSize,portraitSize)
				-- Growler.bar.PortraitFrame.portraitframe:SetPoint("BOTTOMRIGHT",portraitSize,-portraitSize)
				-- Growler.bar.PortraitFrame.portrait:ClearAllPoints()
				-- Growler.bar.PortraitFrame.portrait:SetAllPoints(Growler.bar.PortraitFrame)
				-- SetPortraitTexture(Growler.bar.PortraitFrame.portrait,"pet")
				
				Growler.bar.stablebtn:SetPoint("RIGHT",Growler.bar,"RIGHT",((-Growler.bar.settingsbtn.size) - btnSpacing) * Growler.bar.stablebtn.pos,0 * scale)
				Growler.bar.stablebtn.texture:SetPoint("TOPLEFT",Growler.bar.stablebtn,"TOPLEFT",0 * scale,20 * scale)
				Growler.bar.stablebtn.texture:SetPoint("BOTTOMRIGHT",Growler.bar.stablebtn,"BOTTOMRIGHT",0 * scale,(-2) * scale)
				Growler.bar.lockbtn:SetPoint("RIGHT",Growler.bar,"RIGHT",((-Growler.bar.settingsbtn.size) - btnSpacing) * Growler.bar.lockbtn.pos,0 * scale)
				Growler.bar.lockbtn.texture:SetPoint("TOPLEFT",Growler.bar.lockbtn,"TOPLEFT",0 * scale,0 * scale)
				Growler.bar.lockbtn.texture:SetPoint("BOTTOMRIGHT",Growler.bar.lockbtn,"BOTTOMRIGHT",16 * scale,(-18) * scale)
				Growler.bar.petslotbtn:SetPoint("RIGHT",Growler.bar,"RIGHT",((-Growler.bar.settingsbtn.size) - btnSpacing) * Growler.bar.petslotbtn.pos,0 * scale)
				Growler.bar.petslotbtn.texture:SetPoint("TOPLEFT",Growler.bar.petslotbtn,"TOPLEFT",0 * scale,20 * scale)
				Growler.bar.petslotbtn.texture:SetPoint("BOTTOMRIGHT",Growler.bar.petslotbtn,"BOTTOMRIGHT",0 * scale,(-2) * scale)
				if hasLonewolf then Growler.bar:Hide() else Growler.bar:Show() end
			else
				Growler.msg("Cannot do that in combat.",true)
			end
		end
	end
	
	Growler.CreateSpecButtons = function()
		for spec=1,3 do
			local icon = "Interface\\ICONS\\Ability_Druid_KingoftheJungle"
			if spec == 2 then
				icon = "Interface\\ICONS\\ABILITY_DRUID_DEMORALIZINGROAR"
			elseif spec == 3 then
				icon = "Interface\\ICONS\\Ability_EyeOfTheOwl"
			end
			local specbtn = CreateFrame("Button","GrowlerSpecBtn_" .. tostring(spec), Growler.bar, "SecureActionButtonTemplate")
			Growler.specbtn[spec] = specbtn
			specbtn:Show()
			specbtn:SetFrameLevel(0)
			specbtn.dpsremind = false
			specbtn.lastremind = 4.5
			specbtn.active = (GetSpecialization(false,true) == spec)
			specbtn:SetAttribute("type1","macro")
			specbtn:SetAttribute("macrotext1","/run SetSpecialization(" .. spec .. ",true)")
			specbtn:SetSize(petbtnSize,petbtnSize)
			specbtn:SetAlpha(1)
			specbtn:SetClampedToScreen(true)
			L.SetTexture(icon,specbtn)
			
			specbtn.setTooltipText = function()
				local tt = ""
				local talents = (spec==1 and "Ferocity" or (spec==2 and "Tenacity" or "Cunning"))
				if specbtn.active then
					tt = "|cff888888Your pet's currently active talents: "..talents
				else
					tt = "|rLeft-click: |cffffffffChange your pet's talents to |cff00ff00"..talents.."."
				end
				GameTooltip:SetText(tt)
			end
			specbtn.updateTooltip = function()
				if specbtn.mouseover and GrowlerDB.options.tooltips then
					specbtn.setTooltipText()
				end
			end
			specbtn:SetScript("OnEnter",function(self)
				if GrowlerDB.options.tooltips then
					specbtn.mouseover = true
					GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
					specbtn.setTooltipText()
					GameTooltip:ClearAllPoints()
					GameTooltip:SetPoint("BOTTOMLEFT",specbtn,"TOPLEFT",0,0)
					GameTooltip:Show()
					specbtn.updateTooltip()
				end
			end)
			specbtn:SetScript("OnLeave",function(self)
				if GrowlerDB.options.tooltips then
					specbtn.mouseover = false
					GameTooltip:ClearAllPoints()
					GameTooltip:Hide()
				end
			end)
			
			specbtn:RegisterForClicks("LeftButtonUp")
			specbtn:RegisterForDrag("LeftButton")
			specbtn:SetScript("OnDragStart", function(self) Growler.bar.DragStart(self) end)
			specbtn:SetScript("OnDragStop", function(self) Growler.bar.DragStop(self) end)
			specbtn:SetScript("OnMouseUp", function(self, button)
				if button == "RightButton" then
				elseif button == "MiddleButton" then
				elseif button == "LeftButton" then
				end
			end)
			specbtn.update = function()
				specbtn.lastremind = specbtn.lastremind + 0.5
				local hasPet, isHunterPet = HasPetUI()
				specbtn.active = (GetSpecialization(false,true) == spec)
				if specbtn.active then
					ActionButton_HideOverlayGlow(specbtn)
					specbtn.texture:SetDesaturated(nil)
					specbtn:RegisterForClicks()
				else
					if specbtn.lastremind > 4.5 then
						local inInstance, instanceType = IsInInstance()
						local _, instanceType, _, _, maxPlayers, _, _, _ = GetInstanceInfo()
						if GrowlerDB.options.dpsremind and spec == 1 and IsInGroup() and maxPlayers>1 and InstanceType~="pvp" and hasPet and isHunterPet and not UnitUsingVehicle("player") and not IsMounted() and not IsFlying() and not UnitOnTaxi("player") then
							if not specbtn.dpsremind then
								if GrowlerDB.options.sound then PlaySoundKitID(31752) end
								ActionButton_ShowOverlayGlow(specbtn)
								specbtn.dpsremind = true
							end
							ActionButton_HideOverlayGlow(specbtn)
							ActionButton_ShowOverlayGlow(specbtn)
							if GrowlerDB.options.sound and GrowlerDB.options.reminder then PlaySoundKitID(38326) end
							specbtn.lastremind = 0
						else
							if specbtn.lastremind > 1 then
								ActionButton_HideOverlayGlow(specbtn)
								specbtn.lastremind = 0
							end
						end
					end
					specbtn.texture:SetDesaturated(1)
					specbtn:RegisterForClicks("LeftButtonUp")
				end
			end
		end
	end
	
	Growler.CreatePetButtons = function()
		for slot=1,5 do
			local petIcon, petName, petLevel, petType, petLoyalty = GetStablePetInfo(slot)
			local petbtn = CreateFrame("Button","GrowlerPetBtn_" .. tostring(slot), Growler.bar, "SecureActionButtonTemplate")
			Growler.petbtn[slot] = petbtn
			petbtn:Show()
			petbtn:SetFrameLevel(0)
			petbtn.active = (Growler.GetActivePetSlot() == slot)
			petbtn.haspetinslot = (GetStablePetInfo(slot) ~= nil)
			petbtn.mouseover = false
			petbtn:SetAttribute("type1","macro")
			petbtn:SetAttribute("type2","macro")
			--petbtn:SetAttribute("type3","macro")
			petbtn:SetSize(petbtnSize,petbtnSize)
			petbtn:SetAlpha(1)
			petbtn:SetClampedToScreen(true)
			L.SetTexture(petIcon,petbtn)			
			
			petbtn.setTooltipText = function()
				local hasPet, isHunterPet = HasPetUI()
				local tt = ""
				if petbtn.active then
					tt = "Right-click: |cffffffffDismiss your pet.\n"
				else
					if petbtn.haspetinslot then
						if hasPet and isHunterPet then
							tt = "Click: |cffffffffDismiss your pet.\n"
						else
							tt = "|rLeft-click: |cffffffffSummon |cff00ff00"..tostring(petName).." |cffffffff("..tostring(petType)..")"
						end
					else
						tt = "Right-click: |cffffffffBeast Lore.\n"
						tt = tt .. "\n|rLeft-click: |cffffffffDismiss Pet / Tame Beast."
					end
				end
				GameTooltip:SetText(tt)
			end
			petbtn.updateTooltip = function()
				if petbtn.mouseover and GrowlerDB.options.tooltips then
					petbtn.setTooltipText()
				end
			end
			petbtn:SetScript("OnEnter",function(self)
				if GrowlerDB.options.tooltips then
					petbtn.mouseover = true
					GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
					petbtn.setTooltipText()
					GameTooltip:ClearAllPoints()
					GameTooltip:SetPoint("BOTTOMLEFT",petbtn,"TOPLEFT",0,0)
					GameTooltip:Show()
					petbtn.updateTooltip()
				end
			end)
			petbtn:SetScript("OnLeave",function(self)
				if GrowlerDB.options.tooltips then
					petbtn.mouseover = false
					GameTooltip:ClearAllPoints()
					GameTooltip:Hide()
				end
			end)
			
			petbtn:RegisterForClicks("LeftButtonUp","RightButtonUp")--,"MiddleButtonUp"
			petbtn:RegisterForDrag("LeftButton")
			petbtn:SetScript("OnDragStart", function(self) Growler.bar.DragStart(self) end)
			petbtn:SetScript("OnDragStop", function(self) Growler.bar.DragStop(self) end)
			petbtn:SetScript("OnMouseUp", function(self, button)
				if button == "RightButton" then
				elseif button == "MiddleButton" then
				elseif button == "LeftButton" then
				end
			end)
			petbtn.update = function()
				local spec = GetSpecialization()
				local _, _, _, selected, _, _, _, _  = GetTalentInfo(1, 1, 1)
				local hasLonewolf = (spec == 2 and selected)
				local petIcon, petName, petLevel, petType, petLoyalty = GetStablePetInfo(slot)
				petbtn.haspetinslot = (petIcon ~= nil)
				
				petbtn.updateTooltip(spec,hasLonewolf,petName,petType)
				
				if not InCombatLockdown() then
					petbtn.texture:SetTexture(petIcon)
					if petIcon == nil then
						petbtn.texture:SetTexture(L.GetSpellIcon(1515))
						petbtn:SetAttribute("macrotext1","/cast [nopet,btn:1] Tame Beast; [pet,btn:1] Dismiss Pet")
						petbtn:SetAttribute("macrotext2","/cast [nopet,btn:2] Beast Lore")
						petbtn.texture:SetVertexColor(1,1,1,1)
					elseif (not hasLonewolf and not BixLib.IsExotic(petType)) or spec == 1 then
						--[@pet,nodead,pet:" .. tostring(petName) .. ",btn:1] Misdirection; 
						petbtn:SetAttribute("macrotext1","/cast [nopet,btn:1] Call Pet " .. tostring(slot) .. "; [nopet:" .. tostring(petName) .. ",btn:1] Dismiss Pet")
						petbtn:SetAttribute("macrotext2","/cast [pet,btn:2] Dismiss Pet")
						--petbtn:SetAttribute("macrotext3","/use [pet:" .. tostring(petName) .. ",btn:3] Essence Swapper")
						petbtn.texture:SetVertexColor(1,1,1,1)
					else
						petbtn:SetAttribute("macrotext1","/run Growler.msg('You cannot control exotic pets.',true)")
						petbtn:SetAttribute("macrotext2","/run Growler.msg('You cannot control exotic pets.',true)")
						petbtn.texture:SetVertexColor(1,0.2,0.2,1)
					end
				end
				
				if petbtn.haspetinslot then
					local hasPet, isHunterPet = HasPetUI()
					petbtn.active = (Growler.GetActivePetSlot() == slot)
					if petbtn.active or not (hasPet and isHunterPet) then
						petbtn.texture:SetDesaturated(nil)
					else
						petbtn.texture:SetDesaturated(1)
					end
				end
			end
		end
	end
	
	Growler.CreateButton = function(name,icon,priority,soundID,macro,tooltip,updateHandler)
		if soundID == nil then soundID = 12867 end
		local btnName = "GrowlerBtn_" .. name
		local btn = CreateFrame("Button", btnName, Growler.bar, "SecureActionButtonTemplate")
		Growler.btn[name] = btn
		btn:Show()
		btn.barpos = L.tablesize(Growler.btn)
		btn.name = btnName
		btn.lastupdate = 0
		btn.snooze = -0.5
		btn.active = false
		btn.soundID = soundID
		btn.priority = priority
		btn.tooltip = tooltip
		btn.mouseover = false
		btn:SetAttribute("type1","macro")
		btn:SetAttribute("macrotext1",macro)
		btn:SetSize(btnSize,btnSize)
		btn:SetAlpha(0.2)
		btn:SetClampedToScreen(true)
		L.SetTexture(icon,btn)
		local text = btn:CreateFontString(btnName .. "_text", "OVERLAY", "DialogButtonNormalText")
		text:SetPoint("BOTTOM",btn,"BOTTOM",0,2 * scale)
		text:SetTextHeight(16 * scale)
		btn.text = text
		btn.si = Growler.CreateSI("growl",icon)
		btn.update = function(hasPet, isHunterPet, petDead, petHealth, petHealthMax)
			updateHandler(hasPet, isHunterPet, petDead, petHealth, petHealthMax)
		end
		btn.setTooltipText = function()
			local tt = ""
			if btn.snooze > 0 then
				local mins,secs = L.SecsToMins(btn.snooze)
				tt = "Snoozing: |cff00ff00" 
				if mins > 0 then
					tt = tt .. tostring(mins) .. "m and "
				end
				tt = tt .. tostring(secs) .. "s |cffffffffleft.\n\n"
				tt = tt .. "|rRight-click: |cffffffffStop snoozing.\n"
			end
			if btn.active then
				tt = "Right-click: |cffffffffSnooze for |cff00ff00" .. math.floor(GrowlerDB.options.snooze) .. " |cffffffffseconds.\n"
			end
			if btn.active or btn.snooze > 0 then
				tt = tt .. "\n|rLeft-click:\n"
			else
				tt = tt .. "|cff888888No action required at the moment.\n\n"
				tt = tt .. "|cff888888Right-Click: Snooze for |cff00ff00" .. math.floor(GrowlerDB.options.snooze) .. " |cff888888seconds.\n"
				tt = tt .. "|cff888888Left-click:\n"
			end
			tt = tt .. "|cffffffff" .. btn.tooltip
			GameTooltip:SetText(tt)
		end
		btn.updateTooltip = function()
			if btn.mouseover and GrowlerDB.options.tooltips then
				btn.setTooltipText()
			end
		end
		btn:SetScript("OnEnter",function(self)
			if GrowlerDB.options.tooltips then
				btn.mouseover = true
				GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
				btn.setTooltipText()
				GameTooltip:ClearAllPoints()
				GameTooltip:SetPoint("BOTTOMLEFT",btn,"TOPLEFT",0,0)
				GameTooltip:Show()
				btn.updateTooltip()
			end
		end)
		btn:SetScript("OnLeave",function(self)
			if GrowlerDB.options.tooltips then
				btn.mouseover = false
				GameTooltip:ClearAllPoints()
				GameTooltip:Hide()
			end
		end)
		btn:RegisterForClicks()
		btn:RegisterForDrag("LeftButton")
		btn:SetScript("OnDragStart", function(self) Growler.bar.DragStart(self) end)
		btn:SetScript("OnDragStop", function(self) Growler.bar.DragStop(self) end)
		btn:SetScript("OnMouseUp", function(self, button)
			if button == "RightButton" then
				if btn.active then
					if btn.snooze < 0 then
						btn.snooze = GrowlerDB.options.snooze
						btn.texture:SetVertexColor(0.3,0.9,1,0.5)
						btn.texture:SetDesaturated(1)
					end
				else
					btn.snooze = 0
				end
			elseif button == "MiddleButton" then
			elseif button == "LeftButton" then
				if btn.snooze > 0 then
					local soundID = 37656
					btn.snooze = 0
					btn.active = false
					ActionButton_HideOverlayGlow(btn)
					btn:SetAlpha(0.2)
					if GrowlerDB.options.sound then PlaySoundKitID(soundID) end
				end
				
				if self.active and self == GrowlerBtn_call then
					if GrowlerDB.options.activepetslot == 0 then
						Growler.msg("You do not have an active pet slot saved. Summon a pet and click the save button.",true)
					else
						Growler_PetCalled = 0
					end
				end
			end
		end)
		btn.show = function(icon,sound)
			btn.lastupdate = btn.lastupdate + 0.5
			if sound == nil then sound = btn.soundID end
			if icon ~= nil then
				if btn.texture:GetTexture()~=icon then
					btn.texture:SetTexture(icon)
					btn.si.texture:SetTexture(icon)
					ActionButton_HideOverlayGlow(btn)
					ActionButton_ShowOverlayGlow(btn)
					if GrowlerDB.options.sound then PlaySoundKitID(sound) end
				end
			end
			if not btn.active then
				btn.si.show()
				btn.active = true
				btn:SetAlpha(1)
				ActionButton_ShowOverlayGlow(btn)
				if GrowlerDB.options.sound and (icon == nil or btn.texture:GetTexture()==icon) then PlaySoundKitID(sound) end
				btn:RegisterForClicks("LeftButtonUp")
				btn.lastupdate = 0
			else
				if btn.lastupdate > 4.5 then
					ActionButton_HideOverlayGlow(btn)
					ActionButton_ShowOverlayGlow(btn)
					btn.lastupdate = 0
					if GrowlerDB.options.sound and GrowlerDB.options.reminder then PlaySoundKitID(38326) end
				end
			end
		end
		btn.hide = function(snooze)
			local soundID = 33338
			if btn.active then
				btn.si.hide()
				btn.active = false
				ActionButton_HideOverlayGlow(btn)
				if snooze then
					soundID = 31756
					btn:SetAlpha(0.8)
					btn:RegisterForClicks("LeftButtonUp")
				else
					btn:SetAlpha(0.2)
					btn:RegisterForClicks()
				end
				if GrowlerDB.options.sound then PlaySoundKitID(soundID) end
			end
			if snooze and btn.snooze >= 0 then
				btn.text:SetText(tostring(L.SecsToMins(btn.snooze,true)))
			else
				btn.text:SetText()				
			end
		end
	end
		
	Growler.CreateButtons = function()
		Growler.CreateButton("growl",L.GetSpellIcon(2649),1,nil,
		"/petautocaston Growl\n/petautocastoff [@party1,exists][@raid1,exists] Growl",
		"Turns off Growl autocast when in a group.\nTurns on Growl autocast when alone.",
		function(hasPet, isHunterPet)
			local autocastable, autostate = GetSpellAutocast("Growl")
			local inInstance, instanceType = IsInInstance()
			local _, instanceType, _, _, maxPlayers, _, _, _ = GetInstanceInfo()
			if L.PlayerHasSpell("Growl") and GrowlerDB.options.enable_growl and Growler.PetAvailable and instanceType ~= "pvp" and ((IsInGroup() and autostate==true and maxPlayers > 1) or (((not IsInGroup() or maxPlayers==1) and autostate==false) and GrowlerDB.options.nonparty)) then
				Growler.btn.growl.show()
			else
				Growler.btn.growl.hide()
			end
		end)
		
		Growler.CreateButton("call",L.GetSpellIcon(23498),5,nil,
		"",
		"Calls your saved pet if you have no pet.\nRevives your pet if it's dead.\nHeals your pet if it's low on health.",
		function(hasPet, isHunterPet, petDead, petHealth, petHealthMax)
			if GrowlerDB.options.enable_call and not hasPet and WaitForPet > 1 and not IsFalling() and not UnitUsingVehicle("player") and not IsMounted() and not IsFlying() and not UnitOnTaxi("player") and Growler_PetCalled > 3 then
				Growler.btn.call.show(L.GetSpellIcon(23498),31754)
			else
				if L.PlayerHasSpell("Revive Pet") and GrowlerDB.options.enable_revive and (petDead or (not hasPet and not UnitUsingVehicle("player") and not IsMounted() and not IsFlying() and not UnitOnTaxi("player") and GrowlerDB.options.enable_call and Growler_PetCalled <= 3 and Growler_PetCalled > 1)) then
					Growler.btn.call.show(L.GetSpellIcon(982),38323)
				elseif L.PlayerHasSpell("Mend Pet") and GrowlerDB.options.enable_mend and hasPet and isHunterPet and ((petHealth/petHealthMax) < GrowlerDB.options.mend_threshhold) and L.SpellIsReady("Mend Pet") then
					Growler.btn.call.show(L.GetSpellIcon(136),37656)
				else
					Growler.btn.call.hide()
				end
			end
		end)
		Growler.btn.call.updatemacro = function()
			if not InCombatLockdown() then
				local callpet = ""
				local revivepet = ""
				if GrowlerDB.options.enable_call then
					callpet = "\n/castsequence [nopet] reset=3 Call Pet " .. tostring(GrowlerDB.options.activepetslot) .. ", Revive Pet"
				end
				if GrowlerDB.options.enable_revive or GrowlerDB.options.enable_mend then
					revivepet = "/cast [pet] Revive Pet"
				end
				
				Growler.btn.call:SetAttribute("macrotext1",revivepet .. callpet)
			else
				Growler.msg("Cannot update this setting while in combat.",true)
			end
		end
		Growler.btn.call.updatemacro()
		
		
		Growler.CreateButton("playdead",L.GetSpellIcon(209997),4,34153,
		"/cast Play Dead",
		"Your pet will Play Dead - available when you are feigning death.\nYour pet will Wake Up - available when it's playing dead and you are not.",
		function(hasPet, isHunterPet, petDead)
			local isFeigning = L.UnitHasBuff("player","Feign Death")
			local playsDead = L.UnitHasBuff("pet","Play Dead")
			if L.PlayerHasSpell("Play Dead") and GrowlerDB.options.enable_playdead and hasPet and isHunterPet and not petDead and isFeigning and not playsDead and L.SpellIsReady(209997) then
				Growler.btn.playdead.show(L.GetSpellIcon(209997),34153)
			elseif L.PlayerHasSpell("Play Dead") and GrowlerDB.options.enable_wakeup and hasPet and isHunterPet and not petDead and not isFeigning and playsDead and L.SpellIsReady(210000) then
				Growler.btn.playdead.show(L.GetSpellIcon(210000),34152)
			else
				if not TipPlayDead and UnitLevel("player") == 110 and not L.PlayerHasSpell("Play Dead") then
					Growler.msg("You have not learned the spell 'Play Dead'. Buy the book from your class hall for extra functionality.")
					TipPlayDead = true
				end
				Growler.btn.playdead.hide()
			end
		end)
		
		Growler.CreateButton("passive",L.GetSpellIcon(6462),2,38600,
		"/petpassive",
		"Puts your pet on passive - available when your pet is attacking and you are not.",
		function(hasPet, isHunterPet, petDead)	
			if GrowlerDB.options.enable_passive and hasPet and isHunterPet and not petDead and TimeSinceLastPlayerAttack > 8 and TimeSinceLastPetAttack < 2 then
				for i=1,NUM_PET_ACTION_SLOTS do
					local name, _, _, _, isActive = GetPetActionInfo(i)
					if name == "PET_MODE_PASSIVE" and not isActive then
						if not isActive then
							Growler.btn.passive.show()
						else
							Growler.btn.passive.hide()							
						end
						break
					end
				end
			else
				Growler.btn.passive.hide()
			end
		end)
		
		Growler.CreateButton("assist","Interface\\ICONS\\Ability_hunter_pet_assist",3,38352,
		"/petassist\n/petattack [@target,harm,nodead,exists,combat]",
		"Puts your pet on assist - available when you are attacking and your pet is on passive.",
		function(hasPet, isHunterPet, petDead, elapsed)
			if GrowlerDB.options.enable_assist and hasPet and isHunterPet and not petDead and TimeSinceLastPlayerAttack < 3 and InCombatLockdown() then
				for i=1,NUM_PET_ACTION_SLOTS do
					local name, _, _, _, isActive = GetPetActionInfo(i)
					if name == "PET_MODE_ASSIST" then
						if not isActive then
							Growler.btn.assist.show()
						else
							Growler.btn.assist.hide()							
						end
						break
					end
				end
			else
				Growler.btn.assist.hide()
			end
		end)
	end
		
	Growler.Update = function(self, elapsed)
		local spec = GetSpecialization()
		local _, _, _, selected, _, _, _, _  = GetTalentInfo(1, 1, 1)
		local hasLonewolf = (spec == 2 and selected)
		
		Growler.LastMessageCheck = Growler.LastMessageCheck + elapsed
		TimeSinceLastUpdate = TimeSinceLastUpdate + elapsed
		TimeSinceLastPetAttack = TimeSinceLastPetAttack + elapsed
		TimeSinceLastPlayerAttack = TimeSinceLastPlayerAttack + elapsed
		TimeSinceLastScreenIconUpdate = TimeSinceLastScreenIconUpdate + elapsed
		Growler_PetCalled = Growler_PetCalled + elapsed
		
		if IsFalling() then
			TimeFalling = TimeFalling + elapsed
			if TimeFalling > 0 and TimeFalling < 2 then
				IsJumping = true
			elseif TimeFalling > 2 then
				IsJumping = false
			end
		else
			TimeFalling = 0
		end
		
		if IsFalling() or IsFlying() or IsMounted() or UnitOnTaxi("player") or UnitUsingVehicle("player") then
			WaitForPet = 0
		elseif WaitForPet > 2 then
			WaitForPet = 2
		else
			WaitForPet = WaitForPet + elapsed
		end
		
		if Growler.LastMessageCheck > 2 and GrowlerDB.options.notifygroup then
			Growler.LastMessageCheck = 0
			if Growler.GrowlWasCast and not Growler.GrowlMessageSent then
				Growler.GrowlMessageSent = true
				local myName = UnitName("player")
				local PeopleWhoSpottedGrowl = {myName}
				local i = 2
				for k,v in pairs(Growler.RecievedMessages) do
					if L.split(v,";")[1] == "growl" then PeopleWhoSpottedGrowl[i] = k end
					i = i + 1
				end
				table.sort(PeopleWhoSpottedGrowl)
				if myName == PeopleWhoSpottedGrowl[1] then
					local groupType = "PARTY"
					if IsInRaid() then groupType = "RAID" end
					SendChatMessage("{{Growler alert :: A hunter in the group has not turned off growl -- Download the addon to help manage your pets.}}",groupType)
				end
			end
		end
		
		if TimeSinceLastUpdate > 0.5 then
			TimeSinceLastUpdate = 0
			
			if GrowlerDB.options.enable and Growler.EnteredWorld then 
				if not hasLonewolf and not UnitIsDeadOrGhost("player") then
					if not Growler.bar:IsShown() then Growler.bar.ResetPos() end
					
					local hasPet, isHunterPet = HasPetUI()
					local petHealth = UnitHealth("pet")
					local petHealthMax = UnitHealthMax("pet")
					local petDead = UnitIsDead("pet")
					
					IsJumpingNotFalling = (IsJumping or (not IsFalling() and not IsJumping))
					Growler.PetAvailable = (hasPet and isHunterPet and IsJumpingNotFalling and not IsMounted() and not IsFlying() and not UnitOnTaxi("player") and not UnitUsingVehicle("player"))
					
					for k,btn in pairs(Growler.btn) do
						if btn.snooze >= 0 then
							btn.snooze = btn.snooze - 0.5
							btn.hide(true)
						elseif btn.snooze <= 0 then
							btn.texture:SetVertexColor(1,1,1,1)
							btn.texture:SetDesaturated()
							btn.update(hasPet, isHunterPet, petDead, petHealth, petHealthMax)
						end
						btn.updateTooltip()
					end
					Growler.UpdateSI()
				else
					if Growler.bar:IsShown() then Growler.bar.ResetPos() end
				end
			end
			Growler.bar.update()
		end
	end
end
function Growler_OnLoad(self, event,...)
    self:RegisterEvent("ADDON_LOADED")
end
function Growler_OnEvent(self, event, ...)
	local isHunter = (tostring(UnitClass("player")) == "Hunter")
	if event == "ADDON_LOADED" then
		if ... == "Growler" then
			self:UnregisterEvent("ADDON_LOADED")
			self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
			self:RegisterEvent("CHAT_MSG_ADDON")
			RegisterAddonMessagePrefix("growler")
			Growler_InitiateDB()
			Growler_Setup()
			
			if isHunter then
				self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
				self:RegisterEvent("PET_STABLE_UPDATE")
				self:RegisterEvent("PET_STABLE_SHOW")
				self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
				self:RegisterEvent("CLOSE_WORLD_MAP")
				self:RegisterEvent("PLAYER_ENTERING_WORLD")
				Growler.SetActivePetSlot(true)
				Growler.CreateBar()
				Growler.CreatePetButtons()
				Growler.CreateSpecButtons()
				Growler.CreateButtons()
				S.CreateMainUI()
				self:SetScript("OnUpdate", Growler.Update)
				
				if not Growler.HasVisitedStables() then
					Growler.msg("Visit the stables to use the stable function. (you only have to do this once)",true)
				end
			end
			
			O.CreateOptionsPanel()
		end
	elseif event == "UNIT_SPELLCAST_SUCCEEDED" then
		Growler.CheckForGroupGrowl(...)
	elseif event == "CHAT_MSG_ADDON" then
		Growler.AddonMsgRecieved(...)
	end
	
	if isHunter then
		if event == "COMBAT_LOG_EVENT_UNFILTERED" then
			Growler.CheckCombatLogEvent(...)
		elseif event == "PET_STABLE_SHOW" or event == "PET_STABLE_UPDATE" then
			S.UpdateStables()
		elseif event == "PLAYER_ENTERING_WORLD" then
			Growler.EnteredWorld = true
			S.HideStable()
			Growler.bar.ResetPos()
		end
	end
end
